/*
   atphone.cpp Sample file for sending AT command

   Copyright (C) 2003, 2004, Intel Corporation

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <pthread.h>
#include <errno.h>
#include <termios.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>

#include "dialerimpl.h"
#include "atphone.h"

void *Listening_Thread( void * arg )
{
	atphone * phone;
	int count;
	char response[256];
	char* p;
	int retval;
	fd_set rfds;

	if ( !arg ) {
		printf("Listening_Thread failed.\n");
		pthread_exit(0);
	}

	phone = (atphone*) arg;
	while ( 1 ) {
		FD_ZERO(&rfds);
		FD_SET(phone->serial_fd, &rfds);

		retval = select(phone->serial_fd+1, &rfds, NULL, NULL, NULL);
		if ( retval < 0 ) {
			fprintf( stderr, "error select.\n");
			break;
		}

		if ( FD_ISSET(phone->serial_fd, &rfds) ) {
			memset( response, 0, 256 );
			p = response;
			while ( (count = read( phone->serial_fd, p, response + sizeof(response) - p - 1)) > 0 ) {
				p += count;
			}

			*p= '\0';

#ifdef DEBUG
			printf("Receive the response from Garson: %s@end\n", response );
#endif

			if ( strstr( response, "NO CARRIER") != NULL ) {
				phone->dialer->setCaption( "Mini dialer" );
				phone->state = INIT_STATE;
				phone->dialer->connected = 0;
			} else

			if ( strstr( response, "RING" )!=NULL ) {
				if ( phone->state != INACALL_STATE ) {
					phone->dialer->setCaption("Ring...");
					phone->state = NEWCALL_STATE;
				}
			} else

			if ( strstr( response, "OK" ) !=NULL ) {
				switch ( phone->state ) {
				case INITING_STATE:
					phone->state = INITED_STATE;
					break;
				case DIALING_STATE:
					phone->dialer->InACall();
					break;
				case ANSWERING_STATE:
					phone->dialer->InACall();
					break;
				case HANGINGUP_STATE:
					phone->dialer->Hangedup();
					break;
				default:
					break;
#ifdef DEBUG
#endif
				}
			} else if ( strstr( response, "NO DIALTONE" )!=NULL ) {
				switch( phone->state ) {
				case DIALING_STATE:
					phone->dialer->warning("No dialtone!\n");
					phone->state = INIT_STATE;
					break;
				default:
					phone->dialer->warning("DIAL");
					break;
				}
			} else if ( strstr( response, "BUSY") !=NULL ) {
				switch( phone->state ) {
				case DIALING_STATE:
					phone->state = INIT_STATE;
					break;
				default:
					break;
				}
			} else if ( strstr( response, "ERROR") != NULL ) {
				phone->state = INIT_STATE;
				phone->dialer->connected = 0;
			}
		}
	}
}

atphone::atphone( DialerImpl * dialer )
{
#if 0
	struct sigaction sa;
#endif
	struct termios options;

	this->dialer = dialer;

	serial_fd = open( SERIAL_DEV, O_RDWR | O_NOCTTY | O_NDELAY);
	if ( serial_fd<0 ) {
		perror("open");
	}

	fcntl(serial_fd, F_SETFL, 0);

	tcgetattr(serial_fd, &options);
	memcpy( &old_options, &options, sizeof(options) );

	cfsetispeed(&options, B115200);
	cfsetospeed(&options, B115200);

	options.c_cflag	    |= (CLOCAL|CREAD);
	options.c_lflag	    &= ~(ICANON|ECHO|ECHOE|ISIG);
	options.c_oflag	    &= ~(OPOST);
	options.c_cc[VMIN]   = 0;
	options.c_cc[VTIME]  = 10;
	options.c_cflag &= ~CRTSCTS;

	tcsetattr(serial_fd, TCSANOW, &options);

	this->state = INIT_STATE;

	if ( pthread_create( &listening_thread, NULL, Listening_Thread, (void*)this )<0 ) {
		printf("ERROR; return code from pthread_create() is 0\n");
		exit(-1);
	}
}

atphone::~atphone( void )
{
	tcsetattr(serial_fd, TCSAFLUSH, &old_options);
	close( serial_fd );
}

int atphone::send_command ( char *cmd )
{
	char response[256];
	char *p;
	int count, ret = RESULT_ERROR, retries = 0;

	write( serial_fd, cmd, strlen(cmd) );
	write( serial_fd, CRNL, strlen(CRNL) );

#ifdef DEBUG
	printf("send out command=%s\n", cmd);
#endif

	return ret;
}

int atphone::try_send_command( char* cmd, int Retry_times )
{
	int retVal;
	int retry = 0;

	while ( retry < Retry_times ) {
		retVal = send_command( cmd );
		if ( retVal == RESULT_OK ) {
			break;
		}
		retry ++;
	}

	if ( retry > Retry_times )
		return -1;

	return 0;
}

int atphone::wait_for_ring( void )
{
	char ring[256];
	char * p;
	int count;

	memset( ring, 0, 256*sizeof(char) );
	p = ring;
	while ( (count = read( serial_fd, p, ring + sizeof(ring) - p - 1)) > 0 ) {
		p += count;
		if ( p[-1] =='\n'  || p[-1]=='\r' ) {
#ifdef DEBUG
			printf("This is a NL or CR. 0X%08x\n", p[-1] );
#endif
			break;
		}
	}

	*p= '\0';

#ifdef DEBUG
	fprintf(stderr, "ring = %s", ring );
#endif

	if ( strstr( ring, "RING" )  != NULL ) {
		printf("Get Ring\n");
		return 1;
	}

	return -1;
}

int atphone::dial( const char* phonenumber )
{
	char calling_string[256];
	int result;

	memset( calling_string, 0, 256*sizeof(char) );
	snprintf( calling_string, 255, "ATD%s;", phonenumber );
	result = try_send_command( calling_string, 1 );
	this->state = DIALING_STATE;
	return result;
}

int atphone::init(  )
{
	int result;

	result = try_send_command("AT", 1);
	this->state = INITING_STATE;
	return result;
}

int atphone::hangup( void )
{
	int result;

	result = try_send_command("ATH", 1);
	this->state = HANGINGUP_STATE;
	return result;
}

int atphone::answer( void )
{
	int result;

	result = try_send_command("ATA", 1 );
	this->state = ANSWERING_STATE;
	return result;
}

int atphone::generate_DTMFtone( char* str )
{
	char dtmf_str[256];
	int result;

	memset( dtmf_str, 0, 256*sizeof(char) );
	snprintf( dtmf_str, 255, "AT+VTS=%s", str );
	result = try_send_command( dtmf_str, 1 );
	return result;
}
